@@ -15,10 +15,10 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
-#ifndef _CURSES_IF_H
+#ifndef CURSES_IF_H
 
-#define _CURSES_IF_H
+#define CURSES_IF_H
 
-extern void text_mode (int);
+extern void curses_game_init (void);
 
 #endif /* curses_if.h */
