@@ -25,30 +25,44 @@
 #include "wrappers.h"
 #include "fill.h"
 
+/* A start marked as OPP_START passes the initial checks, since a
+   check to the previous direction has already covered those cells.
+*/
+enum start_type {
+  NORMAL, OPP_START
+};
+
+struct start_buf {
+  enum start_type type;
+  Coord loc;
+};
+
 typedef struct {
-  Coord *start;
+  struct start_buf *start;
   int entries, bufmax;
 } Startbuf;
 
-static void push_start (Coord, Startbuf *);
+static void push_start (const Coord, Startbuf *, const enum
+			start_type);
 static void track_starts (int *, Startbuf *, enum fill_op (Coord),
 			  Coord);
-static Coord pop_start (Startbuf *);
+static struct start_buf pop_start (Startbuf *);
 
 static void
-push_start (loc, list)
-     Coord loc;
+push_start (loc, list, type)
+     const Coord loc;
      Startbuf *list;
+     const enum start_type type;
 {
   if (list->entries == list->bufmax)
     {
       list->bufmax++;
       if (list->entries % CHUNK_SIZE == CHUNK_SIZE-1)
-	list->start = (Coord *) realloc (list->start, (list->bufmax +
-						       CHUNK_SIZE) *
-					 sizeof (Coord));
+	list->start = (struct start_buf *)
+	  realloc (list->start, (list->bufmax + CHUNK_SIZE) * sizeof (Coord));
     }
-  list->start[list->entries++] = loc;
+  list->start[list->entries].type = type;
+  list->start[list->entries++].loc = loc;
 }
 
 static void
@@ -68,13 +82,13 @@ track_starts (tracking, list, check, loc)
       if (check (loc))
 	{
 	  *tracking = 1;
-	  push_start (loc, list);
+	  push_start (loc, list, NORMAL);
 	}
     }
 }
 
-/* Return the last entry on the list.  Decrease list size.  */
-static Coord
+/* Return the last entry on the stack.  Decrease list size.  */
+static struct start_buf
 pop_start (list)
      Startbuf *list;
 {
@@ -88,28 +102,32 @@ fill (loc, check, mark)
      enum fill_op (mark) (Coord);
 {
   Startbuf left, right;
-  int ret_status = 0;
+  enum fill_op ret_status = UNSUCCESSFUL;
+  /* Ensures that both directions are checked.  */
+  int dir_flip = 1;
 
   /* If the starting location already matches the "check" pattern,
      nothing will be done.  */
   if (!check (loc))
-    return 0;
+    return UNSUCCESSFUL;
 
   /* There's got to be at least one "left" entry.  */
   left.entries = left.bufmax = 1;
   right.entries = right.bufmax = 0;
 
-  left.start = (Coord *) my_malloc (CHUNK_SIZE * sizeof (Coord));
-  right.start = (Coord *) my_malloc (CHUNK_SIZE * sizeof (Coord));
-  *left.start = loc;
+  left.start = (struct start_buf *) my_malloc (CHUNK_SIZE * sizeof
+					      (struct start_buf));
+  right.start = (struct start_buf *) my_malloc (CHUNK_SIZE * sizeof
+						(struct start_buf));
+  left.start->loc = loc;
+  left.start->type = NORMAL;
 
   while (1)
     {
       Startbuf *now_dir, *opp_dir;
       int up_start, down_start;
       enum fill_op check_status;
-      int last_dir = 0;
-      Coord location;
+      struct start_buf start_info;
 
       /* Array left_scan has the surrounding cells in order:	632
 	 Array right_scan has the same order, mirrored.		8.1
@@ -124,51 +142,79 @@ fill (loc, check, mark)
 
       up_start = down_start = 0;
 
-      if (left.entries)
+      if (dir_flip)
 	{
-	  scan = left_scan;
-	  now_dir = &left;
-	  opp_dir = &right;
-	  location = pop_start (&left);
-	}
-      else if (right.entries)
-	{
-	  scan = right_scan;
-	  now_dir = &right;
-	  opp_dir = &left;
-	  location = pop_start (&right);
+	  if (left.entries)
+	    {
+	      scan = left_scan;
+	      now_dir = &left;
+	      opp_dir = &right;
+	      start_info = pop_start (&left);
+	    }
+	  else if (right.entries)
+	    {
+	      scan = right_scan;
+	      now_dir = &right;
+	      opp_dir = &left;
+	      start_info = pop_start (&right);
+	    }
+	  else
+	    break;
+	  dir_flip = 0;
 	}
       else
-	break;
+	{
+	  if (right.entries)
+	    {
+	      scan = right_scan;
+	      now_dir = &right;
+	      opp_dir = &left;
+	      start_info = pop_start (&right);
+	    }
+	  else if (left.entries)
+	    {
+	      scan = left_scan;
+	      now_dir = &left;
+	      opp_dir = &right;
+	      start_info = pop_start (&left);
+	    }
+	  else
+	    break;
+	  dir_flip = 1;
+	}
 
-      check_status = check (location);
+      check_status = check (start_info.loc);
       if (check_status == SUCCESSFUL)
 	{
 	  Coord opp_start;
-	  opp_start = add_coord (location, *scan++);
+	  opp_start = add_coord (start_info.loc, scan[0]);
 	  if (check (opp_start))
-	    push_start (opp_start, opp_dir);
-	  
-	  /* Do initial checks for vertical search.  */
-	  track_starts (&up_start, now_dir, check, add_coord
-			(location, *scan++));
-	  track_starts (&up_start, now_dir, check, add_coord
-			(location, *scan++));
-	  track_starts (&down_start, now_dir, check, add_coord
-			(location, *scan++));
-	  track_starts (&down_start, now_dir, check, add_coord
-			(location, *scan++));
+	    push_start (opp_start, opp_dir, OPP_START);
+
+	  if (start_info.type == NORMAL)
+	    {
+	      /* Do initial checks for vertical search.  */
+	      track_starts (&up_start, now_dir, check, add_coord
+			    (start_info.loc, scan[1]));
+	      track_starts (&up_start, now_dir, check, add_coord
+			    (start_info.loc, scan[2]));
+	      track_starts (&down_start, now_dir, check, add_coord
+			    (start_info.loc, scan[3]));
+	      track_starts (&down_start, now_dir, check, add_coord
+			    (start_info.loc, scan[4]));
+
+	    }
 
 	  do
 	    {
-	      if (mark (location) == SUCCESSFUL && ret_status == 0)
-		ret_status = 1;
+	      if (mark (start_info.loc) == SUCCESSFUL && ret_status == 0)
+		ret_status = SUCCESSFUL;
 	      track_starts (&up_start, now_dir, check, add_coord
-			    (location, scan[0]));
+			    (start_info.loc, scan[5]));
 	      track_starts (&down_start, now_dir, check, add_coord
-			    (location, scan[1]));
-	      location = add_coord (location, scan[2]);
-	      check_status = check (location);
+			    (start_info.loc, scan[6]));
+	      start_info.loc = add_coord (start_info.loc, scan[7]);
+	      check_status = check (start_info.loc);
 	    }
 	  while (check_status == SUCCESSFUL);
 	}
