@@ -15,9 +15,9 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
-#ifndef _MAP_H
+#ifndef MAP_H
 
-#define _MAP_H 1
+#define MAP_H 1
 
 #define MASK_PLAYER	31
 #define MASK_PLAIN	63
@@ -31,7 +31,7 @@
 #define MODE_GUE	4
 #define UPDATE_TURN	8
 
-/* A hell will propably break loose if this is changed.  */
+/* A hell will probably break loose if this is changed.  */
 #define MAXPLAYERS 31
 
 typedef struct
@@ -111,10 +111,11 @@ extern int cmp_coord (Coord, Coord);
 extern void set_map (Coord, map_val);
 extern void set_map_bit (Coord, map_val);
 extern void clear_map_bit (map_val);
-extern enum cross_act check_cross (Coord);
+extern enum cross_act check_cross_nocheck (const Coord, const enum cross_act);
+extern enum cross_act check_cross (const Coord);
 extern int toggle_cross (Coord);
 extern int dist_to_capital (int, Coord);
-extern char need_update (void);
+extern char need_update (char *);
 extern void select_next (void);
 extern void clear_crosses (void);
 
